class Shortcode::TemplateBinding

  def initialize(name, configuration, attributes=[], content="", additional_attributes=nil)
    @configuration = configuration
    include_helper_modules
    presenter = Shortcode::Presenter.new name,
                                         configuration,
                                         map_attributes(attributes),
                                         content,
                                         additional_attributes
    @name = name
    @attributes = presenter.attributes
    @content = presenter.content
  end

  # Expose private binding() method for use with erb templates
  def expose_binding
    binding
  end

  private

  attr_reader :configuration

  def map_attributes(attributes)
    hash = {}
    attributes.each { |o| hash[o[:key].to_sym] = o[:value] }
    hash
  end

  def include_helper_modules
    return unless configuration.helpers.any?

    configuration.helpers.each do |helper|
      self.class.send(:include, helper)
    end
  end

end
